/* 
 * File: component.js
 * License: MPL 1.1
 *
 * Portions derived from Adblock
 *
 * Other Original Code (C) 2004 Rich Persaud <dotpeople@yahoo.com>, All Rights Reserved
 *
 */

/*
 * Constants / Globals
 */

const RPXP_COID  			= "@mozilla.org/rpxp;1"							;
const RPXP_CID = Components.ID('{242fab16-2087-40cb-bf7d-606dc7827d1e}')	;

const PREFSERVICE_COID  	= "@mozilla.org/preferences-service;1"			;
const DIRSERVICE_COID  		= '@mozilla.org/file/directory_service;1'		;
const OBSERVER_COID 		= "@mozilla.org/observer-service;1"				;

const ID_RPXP_BRANCH 		=	'rpxp.dailykos.com.'						;


var gIsEnabled 	  =	false						;
var goBlueMembers = {}							;
var goRedMembers  = {}							;

const COLOR_BLUE  = '#1E90FF'					;	// dodgerblue
const COLOR_RED   = '#DC143C'					;	// crimson
const BORDER_BLUE = '1px solid ' + COLOR_BLUE	;
const BORDER_RED  = '1px solid ' + COLOR_RED	;

/* 2004-05-22  oskar  23:19  RJP | create object for component export via XPCOM double-wrapper
 * http://forums.mozillazine.org/viewtopic.php?p=134028#134028
 */

var rpxpWrappedGlobal = 
{
	fSetBlue: function ( pNode )
	{
			   goBlueMembers [ pNode.text ] = 1		;
		delete goRedMembers  [ pNode.text ]			;
		pNode.style.color 		 = COLOR_BLUE 		;
		pNode.style.borderBottom = BORDER_BLUE		;
	},

	fSetRed: function ( pNode )
	{
		delete goBlueMembers [ pNode.text ]			;
			   goRedMembers  [ pNode.text ] = 1		;
		pNode.style.color 		 = COLOR_RED 		;
		pNode.style.borderBottom = BORDER_RED		;
	},

	fSetNeutral: function ( pNode )
	{
		delete goBlueMembers [ pNode.text ]			;
		delete goRedMembers  [ pNode.text ]			;
		pNode.style.color 		 = null				;
		pNode.style.borderBottom = null				;
	},

	fUpdateLinks: function ( pLinks, pUser)
	{
		for ( var i = 0; i < pLinks.length; i++ )
		{
			if ( pLinks[i].text == pUser )
			{
				// FIXME: security - tainted data

				if ( goBlueMembers [ pUser ] )
				{
					pLinks[i].style.color 		 = COLOR_BLUE	;
					pLinks[i].style.borderBottom = BORDER_BLUE	;
				}

				else if ( goRedMembers [ pUser ] )
				{
					pLinks[i].style.color 		 = COLOR_RED	;
					pLinks[i].style.borderBottom = BORDER_RED	;
				}
	
				else
				{
					pLinks[i].style.color 		 = null			;
					pLinks[i].style.borderBottom = null			;
				}
			}
		}
	},
};


// Document Load Observer
var endLoad =
{
	// nsIObserver interface implementation
	observe: function(oSubject, sMessage, sContextUrl) 
	{
		if ( !gIsEnabled) 
			return										;
		if ( ! sContextUrl.match(/dailykos\.com\/?/) ) 
			return										;

		var wnd 	=	oSubject						;
		var oLinks	=	wnd.document.links				;
		var oLink										;

		for ( var i = 0; i < oLinks.length; i++ )
		{
			oLink = oLinks[i]							;

			// FIXME: security - tainted data as hash key
			if ( goBlueMembers [ oLink.text ] )
			{
				oLink.style.color 		 = COLOR_BLUE	;
				oLink.style.borderBottom = BORDER_BLUE	;
			}
			else if ( goRedMembers [ oLink.text ] )
			{
				oLink.style.color 		 = COLOR_RED	;
				oLink.style.borderBottom = BORDER_RED	;
			}
		}
	}, // observe()

	QueryInterface: function(iid) 
	{
		if (!iid.equals(Components.interfaces.nsISupports) &&
			!iid.equals(Components.interfaces.nsISupportsWeakReference) &&
			!iid.equals(Components.interfaces.nsIObserver))
		{
			dump("Rpxp endLoad observer object: QI unknown interface: " + iid + "\n");
			throw Components.results.NS_ERROR_NO_INTERFACE;
		}

		return this;
	},
	
	get wrappedJSObject () 
	{
		return rpxpWrappedGlobal;
	}

} // endLoad()

// Factory object
var factory =
{
	// nsIFactory interface implementation
	 createInstance: function(outer, iid) {
		if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;
		return endLoad;
	},

	// nsIObserver + nsIPrefBranchObserver interface implementation
	observe: function(subject, topic, prefName)
	{
		// subject: [wrapped nsISupports :: nsIPrefBranch], nsIPrefBranchInternal
		
		if (topic == "Rpxp-SavePrefFile") 
		{
			var prefObj = Components.classes[ PREFSERVICE_COID ].getService(Components.interfaces.nsIPrefService);
			prefObj.savePrefFile(null); 		// save prefs to disk
		}
		else if (topic == "Rpxp-PrefChange" && prefName == "FilterChange") 
		{
			rpxpLoadSettings(prefName);
		}
		else if ( topic == "Rpxp-SerializeMembers" )
		{
			rpxpSerializeMembers ();	
		}
		else if ( prefName == "rpxp.dailykos.com.enabled" )
		{
			var prefObj = Components.classes[ PREFSERVICE_COID ].getService(Components.interfaces.nsIPrefService);
			var Branch = prefObj.getBranch( ID_RPXP_BRANCH );
			gIsEnabled =   !Branch.prefHasUserValue("enabled") 
						|| Branch.getBoolPref("enabled"); // default:true
		}
	},

	// nsISupports interface implementation
	QueryInterface: function(iid) {
		if (!iid.equals(Components.interfaces.nsISupports) &&
			!iid.equals(Components.interfaces.nsISupportsWeakReference) &&
			!iid.equals(Components.interfaces.nsIFactory) &&
			!iid.equals(Components.interfaces.nsIObserver))
		{
			dump("Rpxp content policy factory object: QI unknown interface: " + iid + "\n");
			throw Components.results.NS_ERROR_NO_INTERFACE;
		}

		return this;
	},

} // factory


rpxpInit(); // begin initialization

// Initialization and registration
function rpxpInit() 
{
	// Component registration
	if (typeof (Components.classes[ RPXP_COID ]) == 'undefined') 	
	{
		var compMgr = Components.manager.QueryInterface (Components.interfaces.nsIComponentRegistrar);
		compMgr.registerFactory (RPXP_CID, "RPXP Filter",  RPXP_COID , factory);
	}

	// Standard Pref-observer registration
	try 
	{
		var prefInternal = Components.classes[ PREFSERVICE_COID ].getService(Components.interfaces.nsIPrefBranchInternal);
		prefInternal.addObserver("rpxp", factory, true);
	} 
	catch (e) 
	{ dump("Rpxp content policy registration: exception when registering pref observer: " + e + "\n"); }

	// Custom Pref-observer registration
	try {
		var observerService = Components.classes[ OBSERVER_COID ].getService(Components.interfaces.nsIObserverService);
		observerService.addObserver(factory, "Rpxp-PrefChange", true);
		observerService.addObserver(factory, "Rpxp-SerializeMembers", true);
		observerService.addObserver(factory, "Rpxp-SavePrefFile", true);
		// Document Load-observer registration
		observerService.addObserver(endLoad, "EndDocumentLoad", false);
	} catch (e) { dump("RPXP: exception when registering saveprefs observer: " + e + "\n"); }

	// Temp-Dir Cleanup (for prior uninstalls) -- separate, in case there's no default profile, or the user isn't root
	var dirArray = new Array();
	dirArray[0] = "AChrom"; 	// app. chrome magic-key
	dirArray[1] = "UChrm"; 		// profile chrome magic-key
	dirArray[2] = "ProfD"; 		// profile directory magic-key
	dirArray[3] = "ComsD"; 		// component directory magic-key
	for (var i = 0, n ; i < dirArray.length ; i++) 
	{
		try { 
			var dirService = Components.classes[ DIRSERVICE_COID ].getService(Components.interfaces.nsIProperties);
			var currentDir = dirService.get(dirArray[i], Components.interfaces.nsIFile);
			var tempDir = currentDir.clone();
			var uninstalledFile = currentDir.clone();
			
			tempDir.append("rpxp-temp");
			uninstalledFile.append("rpxp-uninstalled");
			n = 1;
			
			// remove temp-dir
			if (tempDir.exists()) tempDir.remove(true);
			
			// remove installer-uninstalled files
			while (uninstalledFile.exists()) {
				uninstalledFile.remove(true);
				uninstalledFile = currentDir.clone();
				uninstalledFile.append("rpxp-uninstalled" + n);
				n++; }
		} catch(e) { /*throw(e);*/ }
	}

	// Load Settings
	rpxpLoadSettings("Init");
}

// Loads the preferences
function rpxpLoadSettings(prefName) 
{
	var prefObj = Components.classes[ PREFSERVICE_COID ].getService(Components.interfaces.nsIPrefService);
	var Branch = prefObj.getBranch( ID_RPXP_BRANCH );

	// Load Bool-prefs -- on init or inidividual-change
	if (prefName && prefName != "FilterChange") 
	{
		gIsEnabled = !Branch.prefHasUserValue("enabled") || Branch.getBoolPref("enabled"); // default:true
		// don't continue, if we're not initializing
		if (prefName != "Init") return;
	}
	
	// Load Member List (on init or list-change)

	// Blue Members
	var list = Branch.prefHasUserValue("blue_members") 
			? Branch.getCharPref("blue_members").split(",") 
			: null;
	if (list != null && list != "") 
	{
		// delete current properties
		for (var i = 0; i < goBlueMembers.length; i++) 
			delete goBlueMembers [ i ];

		// load list into memory
		for (var i = 0; i < list.length; i++) 
			goBlueMembers [ list[i] ] = 1;
	}

	// Red Members
	list = Branch.prefHasUserValue("red_members") 
			? Branch.getCharPref("red_members").split(",") 
			: null;
	if (list != null && list != "") 
	{
		// delete current properties
		for (var i = 0; i < goRedMembers.length; i++) 
			delete goRedMembers [ i ];

		// load list into memory
		for (var i = 0; i < list.length; i++) 
			goRedMembers [ list[i] ] = 1;
	}

} // rpxpLoadSettings()

function rpxpSerializeMembers ()
{
   	var prefObj = Components.classes[ PREFSERVICE_COID ].getService(Components.interfaces.nsIPrefService);
   	var Branch = prefObj.getBranch( ID_RPXP_BRANCH );

	// join hash keys (not values - can't use join operator)

	var blueMembers = '';
	for ( var i in goBlueMembers )
		blueMembers += i + ',';

	var redMembers  = '';
	for ( var i in goRedMembers )
		redMembers += i + ',';

    try 
	{
    	Branch.setCharPref("blue_members", blueMembers.substr(0, blueMembers.length-1) );
    	Branch.setCharPref("red_members",  redMembers.substr(0 , redMembers.length-1 ) );
    	prefObj.savePrefFile(null); // save the prefs to disk 
	} 
	catch(e) {}

} // rpxpSerializeMembers()


